package com.yakindu.ea.examples.plugin.handlers;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;

import com.yakindu.bridges.ea.core.EAResource;
import com.yakindu.bridges.ea.core.ui.utils.EAResourceUIUtils;
import com.yakindu.ea.examples.plugin.EAExamplePlugin;
import com.yakindu.ea.examples.plugin.generators.HtmlTreeGenerator;

public class EAExampleHandler extends AbstractHandler {

	/**
	 * This handler is registered for the context menu of all eap-files in the
	 * workspace.
	 */
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		final Shell shell = HandlerUtil.getActiveShell(event);

		final IFile eapFile = getSelectedEapFile(event);
		if (eapFile != null) {
			EAResource eaResource = null;
			try {
				// load resource silent (without error reporting)
				eaResource = EAResourceUIUtils.loadEAResourceSilentWithProgressMonitorDialog(shell, eapFile, true);

				processEAResource(shell, eaResource);

			} catch (InterruptedException e) {
				// progress monitor was interrupted - nothing to do.
			} catch (Exception e) {
				EAExamplePlugin.logError("Failed to load file: " + eapFile, e);
			} finally {
				if (eaResource != null && eaResource.isLoaded()) {
					try {
						eaResource.unload();
					} catch (Exception e) {
						// silent fail
					}
				}
			}
		}
		return null;
	}

	private void processEAResource(Shell shell, EAResource eaResource) {
		/*-
		 * Now we can do something with the loaded model.
		 * Some examples:
		 *  - generate code
		 *  - transform to some other model
		 *  - calculate metrics
		 *  - verify constraints
		 *
		 * Here we create a dynamic svg tree embedded in an html file.
		 */
		final long startTime = System.currentTimeMillis();

		try {
			final IFile file = new HtmlTreeGenerator(eaResource).generate();

			final long generationTime = System.currentTimeMillis() - startTime;
			final String msg = String.format("Generation finished in %d ms:\n%s", //
					generationTime, file.getFullPath().toString());

			final int action = new MessageDialog(shell, "Generation successful", null, msg, //
					MessageDialog.INFORMATION, new String[] { "OK, Thanks!", "Open file" }, 1).open();
			if (action == 1) {
				openBrowser(file);
			}
		} catch (Exception e) {

			EAExamplePlugin.logError("Failed to generate HTML file(s) for: " + eaResource.getURI(), e);
			MessageDialog.openError(shell, "Generation failed", "Generation failed with a "
					+ e.getClass().getSimpleName() + "\nPlease see Error Log view for details.");
		}
	}

	private void openBrowser(IFile file) {
		final IPath location = file.getLocation();
		if (location != null) {
			try {
				final URL url = location.toFile().toURI().toURL();
				PlatformUI.getWorkbench().getBrowserSupport().getExternalBrowser().openURL(url);
			} catch (PartInitException | MalformedURLException e) {
				throw new RuntimeException("Failed to open generated file: " + file.getFullPath());
			}
		}
	}

	private IFile getSelectedEapFile(ExecutionEvent event) {
		final IStructuredSelection structuredSelection = HandlerUtil.getCurrentStructuredSelection(event);
		final Object firstElement = structuredSelection.getFirstElement();
		if (firstElement instanceof IFile) {
			final IFile selectedFile = (IFile) firstElement;
			if (EAResource.FILE_EXTENSION.equals(selectedFile.getFileExtension())) {
				return selectedFile;
			}
		}
		return null;
	}
}
